/*****************************************************************************
 * File:			comm.c
 * Description:		Llibreria de comunicació
 *
 * Author:			Sergio Gómez
 *****************************************************************************/

/* FreeRTOS Includes */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

/* LPC Includes */
#include "LPC17xx.h"
#include "type.h"

/* Library Includes */
#include "config.h"
#include "debug.h"
#include "commonUtils.h"
#include "uartio.h"
#include "wifly.h"
#include "comm.h"

#include <string.h>		//Required for: strlen
#include <stdio.h>		//Required for: sprintf
#include <stdlib.h>		//Required for: atoi, atof...

static char buffSend[BUFSENDMAX];		//Buffer per a enviar comandes
static char buffRead[BUFSIZE_COMM];		//Buffer de recepció

xSemaphoreHandle xMutexComm;			//Mútex per a la llibreria de comunicació

/*****************************************************************************
 * Function:		commSrvInit
 * Description:		Configura els pàremetres de la connexió amb el servidor i
 * 					crea el semàfor d'exclusió mútua.
 *
 * @param[in]		HostType	Format en el que passem el host: Nom DNS (0) o IP (1)
 * @param[in]		Host		IP o Nom DNS del servidor remot
 * @param[in]		Port		Port del servidor
 *
 * @return			None
 *****************************************************************************/
void commSrvInit(uint8_t HostType, char *Host, int Port){

	wiflySetTCPConfig(HostType, Host, Port);
	xMutexComm = xSemaphoreCreateMutex();
}

/*****************************************************************************
 * Function:		commSendTemp
 * Description:		Envia una mesura de temperatura al servidor.
 * 					1) Obre una connexió socket TCP
 * 					2) Envia la dada de temperatura
 * 					3) Tanca la connexió socket TCP
 *
 * @param[in]		temp	Temperatura a enviar (float)
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
uint8_t commSendTemp(float temp){

	uint8_t check = FALSE;

	xSemaphoreTake(xMutexComm, portMAX_DELAY);
	{
		sprintf(buffSend, FORMAT_SEND_TEMP, temp);

		if (wiflyTCPOpen()){ 								//Open TCP Connection
			wiflySendDataSocket(buffSend);					//Send Data
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);	//Wait for data response
			wiflySendDataSocket(CLOSETCP_COMMAND);			//Close TCP Connection

			if(strstr(buffRead, RECEIVED_OK))
				check = TRUE;

		} else {
			wiflyReportConnFailure();						//Report communication error
			#ifdef DEBUG_COMM_ERROR
				debug("[commSendTemp] Error opening TCP Connection");
			#endif
		}

	} xSemaphoreGive(xMutexComm);

	return check;
}

/*****************************************************************************
 * Function:		commGetLastUpdate
 * Description:		Obté la data hora i de la darrera modificació dels
 * 					paràmetres de configuració dels que disposa el servidor
 *
 * @return			Valor TIMESTAMP (int) de la darrera modificació
 *****************************************************************************/
int commGetLastUpdate(void){

	xSemaphoreTake(xMutexComm, portMAX_DELAY);
	{
		if (wiflyTCPOpen()){								//Open TCP Connection
			wiflySendDataSocket("GETLASTUPDATE");			//Send Data
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);	//Wait for data response
			wiflySendDataSocket(CLOSETCP_COMMAND);			//Close TCP Connection

		} else {
			wiflyReportConnFailure();						//Report communication error
			#ifdef DEBUG_COMM_ERROR
				debug("[commGetLastUpdate] Error opening TCP Connection");
			#endif
		}

	} xSemaphoreGive(xMutexComm);

	return atoi(buffRead);
}

/*****************************************************************************
 * Function:		commUpdateConfig
 * Description:		Envia una mesura de temperatura al servidor.
 * 					1) Obre una connexió socket TCP
 * 					2) Actualitza les dades de configuració
 * 					3) Tanca la connexió socket TCP
 *
 * @param[out]		ServerConfig	Variable tipus 'Config' (struct) on es
 * 									guardaran les dades actualitzades
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
uint8_t commUpdateConfig(Config *ServerConfig){

	uint8_t check = FALSE;

	xSemaphoreTake(xMutexComm, portMAX_DELAY);
	{
		if (wiflyTCPOpen()){								//Open TCP Connection

			wiflySendDataSocket("GETTEMPMAX");
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);
			ServerConfig->TempMax = atof(buffRead);
			#ifdef DEBUG_COMM_INFO
				debug("Obtinguda temperatura màxima");
				debug(buffRead);
			#endif

			wiflySendDataSocket("GETTEMPMIN");
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);
			ServerConfig->TempMin = atof(buffRead);
			#ifdef DEBUG_COMM_INFO
				debug("Obtinguda temperatura minima");
				debug(buffRead);
			#endif

			wiflySendDataSocket("GETTEMPINTERVAL");
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);
			ServerConfig->TempInterval = atoi(buffRead);
			#ifdef DEBUG_COMM_INFO
				debug("Obtingut interval de mesures");
				debug(buffRead);
			#endif

			wiflySendDataSocket("GETMAILSERVER");
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);
			sprintf(ServerConfig->MailServer, "%s", buffRead);
			#ifdef DEBUG_COMM_INFO
				debug("Obtingut Mail Server");
				debug(buffRead);
			#endif

			wiflySendDataSocket("GETMAILPORT");
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);
			ServerConfig->MailPort = atoi(buffRead);
			#ifdef DEBUG_COMM_INFO
				debug("Obtingut Mail Port");
				debug(buffRead);
			#endif

			wiflySendDataSocket("GETMAILUSER");
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);
			sprintf(ServerConfig->MailUser,"%s", buffRead);
			#ifdef DEBUG_COMM_INFO
				debug("Obtingut Mail User");
				debug(buffRead);
			#endif

			wiflySendDataSocket("GETMAILPASS");
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);
			sprintf(ServerConfig->MailPass,"%s", buffRead);
			#ifdef DEBUG_COMM_INFO
				debug("Obtingut Mail Password");
				debug(buffRead);
			#endif

			wiflySendDataSocket("GETMAILFROM");
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);
			sprintf(ServerConfig->MailFrom, "%s", buffRead);
			#ifdef DEBUG_COMM_INFO
				debug("Obtingut Mail From");
				debug(buffRead);
			#endif

			wiflySendDataSocket("GETMAILTO");
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);
			sprintf(ServerConfig->MailTo,"%s", buffRead);
			#ifdef DEBUG_COMM_INFO
				debug("Obtingut Mail To");
				debug(buffRead);
			#endif

			wiflySendDataSocket("GETSOUNDALARM");
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);
			ServerConfig->SoundAlarm = atoi(buffRead);
			#ifdef DEBUG_COMM_INFO
				debug("Obtingut estat SoundAlarm");
				debug(buffRead);
			#endif

			wiflySendDataSocket("GETLASTUPDATE");
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);
			ServerConfig->LastUpdate = atoi(buffRead);
			#ifdef DEBUG_COMM_INFO
				debug("Obtinguda data de la darrera actualitzacio");
				debug(buffRead);
			#endif

			check = TRUE;

			wiflySendDataSocket(CLOSETCP_COMMAND);			//Close TCP Connection

		} else {
			wiflyReportConnFailure();						//Report communication error
			#ifdef DEBUG_COMM_ERROR
				debug("[commUpdateConfig] Error opening TCP Connection");
			#endif
		}

	} xSemaphoreGive(xMutexComm);

	return check;

}

/*****************************************************************************
 * Function:		commSendMail
 * Description:		Envia un correu electrònic utilitzant un servidor SMTP
 * 					1) Obté les dades del servidor actualment configurat
 * 					2) Configura les dades del servidor SMTP
 * 					3) Envia mail, amb comunicació SMTP/TCP
 * 					4) Reestableix la configuració del servidor anterior
 *
 * @param[in]		HostSMTP	Servidor SMTP
 * 					PortSMTP	Port del servidor SMTP
 * 					User		Nom d'usuari per l'Autenticació
 * 					Password	Contrasenya per l'Autenticació
 * 					MailDest	Mail del destinatari
 * 					Subject		Assumpte del missatge
 * 					Body		Cos del missatge
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
uint8_t commSendMail(char *HostSMTP, int PortSMTP, char *User, char *Password, char *MailDest, char *Subject, char *Body){

	uint8_t check = FALSE;

	char UserEncoded[255];
	char PasswordEncoded[255];
	char savedIP[20];
	char savedHost[255];
	uint32_t savedPort;
	int lenghtUser = strlen(User);
	int lenghtPassword = strlen(Password);

	//0. Conversió del nom d'usuari i password a BASE64
	Base64encode(UserEncoded, User, lenghtUser);
	Base64encode(PasswordEncoded, Password, lenghtPassword);

	xSemaphoreTake(xMutexComm, portMAX_DELAY);
	{
		//1. Obtenim la IP/HOST i el PORT del servidor actualment configurat al WiFly
		wiflyGetTCPConfig(savedIP, savedHost, &savedPort);

		//2. Configurem WiFly amb nous paràmetres i ens connectem al servidor SMTP
		wiflySetTCPConfig(HOSTDNS, HostSMTP, PortSMTP);

		if (wiflyTCPOpen()){								//Open TCP Connection
			wiflySendDataSocket("EHLO");
			wiflySendDataSocket("AUTH LOGIN");
			wiflySendDataSocket(UserEncoded);
			wiflySendDataSocket(PasswordEncoded);
			wiflySendDataSocket("MAIL FROM:sensor@tfc.sergiogomez.es");
			sprintf(buffSend, "RCPT TO: %s", MailDest);
			wiflySendDataSocket(buffSend);
			wiflySendDataSocket("DATA");
			sprintf(buffSend, "Subject: %s", Subject);
			wiflySendDataSocket(buffSend);
			wiflySendDataSocket(Body);
			wiflySendDataSocket(".");
			wiflyGetDataSocket(buffRead, TIME_GET_DATA);	//Wait for data response
			wiflySendDataSocket("quit");
			vDelayMs(ACTIVE, TIME_GET_DATA);				//Esperem a que es tanqui correctament la connexió

			if(strstr(buffRead, MAIL_QUEUED))				//Comprovem resposta del servidor SMTP
				check = TRUE;

		} else {
			wiflyReportConnFailure();						//Report communication error
			#ifdef DEBUG_COMM_ERROR
				debug("[commSendMail] Error opening TCP Connection");
			#endif
		}

		//4. Reestablim configuració anterior
		if (strlen(savedHost)>1){
			wiflySetTCPConfig(HOSTDNS, savedHost, savedPort);
		} else {
			wiflySetTCPConfig(HOSTIP, savedIP, savedPort);
		}

	} xSemaphoreGive(xMutexComm);

	return check;
}
