/*****************************************************************************
 * File:			gpio.c
 * Description:		Driver for General Purpose Input/Output ports on LPC1769
 *
 * Author:			Sergio Gómez
 *****************************************************************************/

/* LPC Includes */
#include "LPC17xx.h"
#include "type.h"

#include "gpio.h"


/*****************************************************************************
 * Function:		GPIOInitPin
 * Description:		Inicialitza un PIN per ser usat com a GPIO,
 * 					posant '00' (GPIO) en el registre corresponent
 *
 * Documentation: 	Veure Datasheet UM10360, apartat 8.5
 *
 * @param[in]		PINSELNumber		Número de registre PINSEL
 * @param[in]		PINSELFirstBit		Primer bit del registre PINSEL que controla el PIN
 *
 * @return			None
 *****************************************************************************/
void GPIOInitPin(uint8_t PINSELNumber, uint8_t PINSELFirstBit){

	switch(PINSELNumber){
	case 0:
		LPC_PINCON->PINSEL0	&= (~( 3 << PINSELFirstBit));
		break;

	case 1:
		LPC_PINCON->PINSEL1	&= (~( 3 << PINSELFirstBit));
		break;

	case 2:
		LPC_PINCON->PINSEL2	&= (~( 3 << PINSELFirstBit));
		break;

	case 3:
		LPC_PINCON->PINSEL3	&= (~( 3 << PINSELFirstBit));
		break;

	case 4:
		LPC_PINCON->PINSEL4	&= (~( 3 << PINSELFirstBit));
		break;

	case 5:
		LPC_PINCON->PINSEL5	&= (~( 3 << PINSELFirstBit));
		break;

	case 6:
		LPC_PINCON->PINSEL6	&= (~( 3 << PINSELFirstBit));
		break;

	case 7:
		LPC_PINCON->PINSEL7	&= (~( 3 << PINSELFirstBit));
		break;

	case 8:
		LPC_PINCON->PINSEL8	&= (~( 3 << PINSELFirstBit));
		break;

	case 9:
		LPC_PINCON->PINSEL9	&= (~( 3 << PINSELFirstBit));
		break;

	case 10:
		LPC_PINCON->PINSEL10 &= (~( 3 << PINSELFirstBit));
		break;
	}
}

/*****************************************************************************
 * Function:		GPIOSetPinDirection
 * Description:		Configura la direcció (INPUT o OUTPUT) d'un PIN
 *
 * @param[in]		GPIOPort		Port GPIO
 * @param[in]		Bit				Bit del port
 * @param[in]		PortDirection	INPUT(0) o OUTPUT(1)
 *
 * @return			None
 *****************************************************************************/
void GPIOSetPinDirection(uint8_t GPIOPort, uint8_t Bit, uint8_t PortDirection){

	switch(PortDirection){
	case 0:	//Input
		LPC_GPIO[GPIOPort]->FIODIR |= ( 0 << Bit );

	case 1:	//Output
		LPC_GPIO[GPIOPort]->FIODIR |= ( 1 << Bit );
	}
}

/*****************************************************************************
 * Function:		GPIOSetPinHigh
 * Description:		Posa un PIN a High
 *
 * @param[in]		GPIOPort	Port GPIO
 * @param[in]		Bit			Bit del port
 *
 * @return			None
 *****************************************************************************/
void GPIOSetPinHigh(uint8_t GPIOPort, uint8_t Bit){

	LPC_GPIO[GPIOPort]->FIOSET = ( 1 << Bit );
}

/*****************************************************************************
 * Function:		GPIOSetPinLow
 * Description:		Posa un PIN a Low
 *
 * @param[in]		GPIOPort	Port GPIO
 * @param[in]		Bit			Bit del port
 *
 * @return			None
 *****************************************************************************/
void GPIOSetPinLow(uint8_t GPIOPort, uint8_t Bit){

	LPC_GPIO[GPIOPort]->FIOCLR = ( 1 << Bit );
}

/*****************************************************************************
 * Function:		GPIOSetPinState
 * Description:		Obté l'estat d'un PIN (si està a HIGH o a LOW)
 *
 * @param[in]		GPIOPort	Port GPIO
 * @param[in]		Bit			Bit del port
 *
 * @return			1, en cas que el PIN estigui a HIGH
 * 					0, en cas que el PIn estigui a LOW
 *****************************************************************************/
uint8_t GPIOGetPinState(uint8_t GPIOPort, uint8_t Bit){

	unsigned long ulBitState;

	/* Obtain the current Port state. */
	ulBitState = LPC_GPIO[GPIOPort]->FIOPIN;

	if (ulBitState & (1 << Bit)) {
		//PIN is high
		return 1;
	} else {
		//PIN is low
		return 0;
	}

}
