/*****************************************************************************
 * File:			uartio.c
 * Description:		Llibreria UART I/O
 * 					Permet llegir i escriure pel port UART (0, 1 o 3)
 *
 * Author:			Sergio Gómez
 *****************************************************************************/

/* FreeRTOS Includes */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

/* LPC Includes */
#include "LPC17xx.h"
#include "type.h"

/* Library Includes */
#include "config.h"
#include "debug.h"
#include "uart.h"
#include "uartio.h"

#include <string.h>		//Required for: strlen
#include <stdio.h>		//Required for: sprintf

static char buffSend[BUFSIZE_UART];		//Buffer per enviar text per la UART
xSemaphoreHandle xMutexUARTIO;			//Mútex per al driver UARTIO

/*****************************************************************************
 * Function:		UARTIOInit
 * Description:		Inicialitza el port UART amb el baudrate especificat
 * 					i crea el semàfor Mútex.
 *
 * @param[in]		UARTPort	Port UART a inicialitzar (0, 1 o 3)
 * @param[in]		BaudRate	BaudRate
 *
 * @return			None
 *****************************************************************************/
void UARTIOInit(uint32_t UARTPort, uint32_t BaudRate){

	 UARTInit(UARTPort, BaudRate);
	 xMutexUARTIO = xSemaphoreCreateMutex();
}

/*****************************************************************************
 * Function:		UARTIOPrintText
 * Description:		Imprimeix un text pel port UART indicat.
 *
 * @param[in]		UARTPort	Port UART (0, 1 o 3)
 * @param[in]		Text		Punter a la cadena de text que es vol imprimir
 *
 * @return			None
 *****************************************************************************/
void UARTIOPrintText(uint32_t UARTPort, char *Text){

	xSemaphoreTake(xMutexUARTIO, portMAX_DELAY);
	{
		UARTSend(UARTPort, (uint8_t *)Text, strlen(Text));
	}
	xSemaphoreGive(xMutexUARTIO);

}

/*****************************************************************************
 * Function:		UARTIOPrintTextNum
 * Description:		Imprimeix un text i un número enter pel port UART indicat.
 *
 * @param[in]		UARTPort	Port UART (0, 1 o 3)
 * @param[in]		Text		Punter a la cadena de text que es vol imprimir
 * @param[in]		Num			Número enter que es vol imprimir
 *
 * @return			None
 *****************************************************************************/
void UARTIOPrintTextNum(uint32_t UARTPort, char *Text, int32_t Num){

	xSemaphoreTake(xMutexUARTIO, portMAX_DELAY);
	{
		sprintf(buffSend, "%s %i", Text, Num);
		UARTSend(UARTPort, (uint8_t *)buffSend, strlen(buffSend));
	}
	xSemaphoreGive(xMutexUARTIO);

}

/*****************************************************************************
 * Function:		UARTIOPrintTextCRLF
 * Description:		Imprimeix un text pel port UART indicat
 * 					i afegeix CRLF (retorn de carro + salt de línia).
 *
 * @param[in]		UARTPort	Port UART (0, 1 o 3)
 * @param[in]		Text		Punter a la cadena de text que es vol imprimir
 *
 * @return			None
 *****************************************************************************/
void UARTIOPrintTextCRLF(uint32_t UARTPort, char *Text){

	xSemaphoreTake(xMutexUARTIO, portMAX_DELAY);
	{
		sprintf(buffSend, "%s\r\n", Text);
		UARTSend(UARTPort, (uint8_t *)buffSend, strlen(buffSend));
	}
	xSemaphoreGive(xMutexUARTIO);
}

/*****************************************************************************
 * Function:		UARTIOPrintTextNumCRLF
 * Description:		Imprimeix un text i un número enter pel port UART indicat
 * 					i afegeix CRLF (retorn de carro + salt de línia).
 *
 * @param[in]		UARTPort	Port UART (0, 1 o 3)
 * @param[in]		Text		Punter a la cadena de text que es vol imprimir
 * @param[in]		Num			Número enter que es vol imprimir
 *
 * @return			None
 *****************************************************************************/
void UARTIOPrintTextNumCRLF(uint32_t UARTPort, char *Text, int32_t Num){

	xSemaphoreTake(xMutexUARTIO, portMAX_DELAY);
	{
		sprintf(buffSend, "%s %i\r\n", Text, Num);
		UARTSend(UARTPort, (uint8_t *)buffSend, strlen(buffSend));
	}
	xSemaphoreGive(xMutexUARTIO);
}

/*****************************************************************************
 * Function:		UARTIOScanf
 * Description:		Llegeix el que arriba pel port UART indicat.
 *
 * @param[in]		UARTPort	Port UART (0, 1 o 3)
 * @param[out]		string		Text obtingut pel port UART
 * @param[in]		timeout		Temps d'espera abans de llegir
 *
 * @return			Nombre de caràcters llegits
 *****************************************************************************/
uint32_t UARTIOScanf(uint32_t UARTPort, char* string, uint32_t timeout){

	int count; //Nombre de caràcters rebuts pel port UART

	switch (UARTPort){
	case 0:	//UART0
		count = UART0scanf(string, timeout);
		break;

	case 1:	//UART1
		count = UART1scanf(string, timeout);
		break;

	case 3:	//UART3
		count = UART3scanf(string, timeout);
		break;
	}

	return count;
}
