/*****************************************************************************
 * File:			wifly.c
 * Description:		Driver WiFly RN-XV
 *
 * Author:			Sergio Gómez
 *****************************************************************************/

/* FreeRTOS Includes */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

/* LPC Includes */
#include "LPC17xx.h"
#include "type.h"

/* Library Includes */
#include "config.h"
#include "debug.h"
#include "uart.h"
#include "uartio.h"
#include "wifly.h"
#include "commonUtils.h"

#include <string.h>		//Required for: strlen
#include <stdio.h>		//Required for: sprintf
#include <stdlib.h>		//Required for: atoi, atof...

/* Private Functions */
uint8_t wiflyCMDModeOn();
uint8_t wiflyCMDModeOff();
uint8_t wiflySendCommand(char* command, char *success, uint32_t timeout);
uint8_t wiflySendCommandCRLF(char* command, char *success, uint32_t timeout);

volatile uint32_t connFailure = 0;		//Variable per controlar la connexió
static char buffSend[BUFSENDMAX];		//Buffer per a enviar comandes
static char buffRead[BUFSIZE_WIFLY];	//Buffer de recepció

xSemaphoreHandle xMutexWifly;			//Mútex per al driver WiFly


/*****************************************************************************
 * Function:		wiflyInit
 * Description:		Inicialitza el port UART del WiFly amb el baudrate indicat,
 * 					fa un reset al WiFly i crea el semàfor d'exclusió mútua.
 *
 * @param[in]		BaudRate	BaudRate
 *
 * @return			None
 *****************************************************************************/
void wiflyInit(uint32_t BaudRate){

	UARTInit(UART_WIFLY, BaudRate);
	vDelayMs(ACTIVE, 3000);
	wiflyReset();

	xMutexWifly = xSemaphoreCreateMutex();
}

/*****************************************************************************
 * Function:		wiflyReset
 * Description:		Realitza un RESET al WiFly
 *
 * @return			None
 *****************************************************************************/
void wiflyReset(){

	// Initialize pin
	LPC_PINCON->PINSEL1 &= ( ~( 3 << 12 ) );
	LPC_GPIO0->FIODIR |= ( 1 << RST_WIFLY_PIN );

	// it made reset at Wifly
	LPC_GPIO0->FIOCLR = 1 << RST_WIFLY_PIN;
	vDelayMs(ACTIVE, 1); // min 160us
	LPC_GPIO0->FIOSET = 1 << RST_WIFLY_PIN;

	vDelayMs(ACTIVE, 5000);
}

/*****************************************************************************
 * Function:		wiflyConnectSSID
 * Description:		Associa el WiFly a una xarxa
 *
 * @param[in]		SSID		Nom SSID de la xarxa
 * @param[in]		AuthType	Tipus de seguretat: Open(0), WEP(1), WPA(2)
 * @param[in]		Pass		Contrasenya
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
uint8_t wiflyConnectSSID(char* SSID, uint32_t AuthType, char* Pass){

	uint8_t check = FALSE;

	xSemaphoreTake(xMutexWifly, portMAX_DELAY);
	{
		wiflyCMDModeOn();	//Entra en mode comandes

		sprintf(buffSend, "set wlan ssid %s", SSID);
		wiflySendCommandCRLF(buffSend, "AOK", TIME_SET);

		switch (AuthType) {
		case 0:	//Open
			break;

		case 1:	//WEP
			sprintf(buffSend, "set wlan key %s", Pass);
			wiflySendCommandCRLF(buffSend, "AOK", TIME_SET);
			break;

		case 2:	//WPA
			sprintf(buffSend, "set wlan phrase %s", Pass);
			wiflySendCommandCRLF(buffSend, "AOK", TIME_SET);
			break;
		}

		sprintf(buffSend, "join %s", SSID);
		check = wiflySendCommandCRLF(buffSend, "Associated", TIME_CONNECT);

		wiflyCMDModeOff();	//Surt del mode comandes
	}
	xSemaphoreGive(xMutexWifly);

	return check;
}

/*****************************************************************************
 * Function:		wiflyPing
 * Description:		Realiza un ping, enviant 1 paquet al host especificat
 *
 * @param[in]		Host
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
uint8_t wiflyPing(char* Host){

	uint8_t check = FALSE;

	xSemaphoreTake(xMutexWifly, portMAX_DELAY);
	{
		wiflyCMDModeOn();	//Entra en mode comandes

		sprintf(buffSend, "ping %s 1", Host);
		check = wiflySendCommandCRLF(buffSend, "time=", TIME_PING);

		wiflyCMDModeOff();	//Surt del mode comandes
	}
	xSemaphoreGive(xMutexWifly);

	return check;
}

/*****************************************************************************
 * Function:		wiflyReportConnFailure
 * Description:		Funció que és cridada quan es detecta un error de connectivitat.
 * 					Quan el nombre d'errors arriba al màxim definit per CONN_FAILURE_MAX,
 * 					es guarda la configuració actual i es fa un reset al WiFly.
 *
 * @return			None
 *****************************************************************************/
void wiflyReportConnFailure(){

	connFailure++;

	if (connFailure == CONN_FAILURE_MAX){
		xSemaphoreTake(xMutexWifly, portMAX_DELAY);
		{
			debug("*** Reiniciem WiFly ***");
			wiflySendCommandCRLF("save", "", TIME_SET);	//Guardem la configuració actual
			wiflyReset();								//Reiniciem el WiFly
			connFailure = 0;
		}
		xSemaphoreGive(xMutexWifly);
	}
}

/*****************************************************************************
 *                               GET FUNCTIONS
 *****************************************************************************/
/*****************************************************************************
 * Function:		wiflyGetIP
 * Description:		Obté la IP del mòdul WiFly
 *
 * @param[out]		Punter de la variable char on es guardarà la IP
 *
 * @return			None
 *****************************************************************************/
void wiflyGetIP(char* string){

	xSemaphoreTake(xMutexWifly, portMAX_DELAY);
	{
		wiflyCMDModeOn();	//Entra en mode comandes

		UARTIOPrintTextCRLF(UART_WIFLY, "get ip");
		UARTIOScanf(UART_WIFLY, buffRead, TIME_GET);
		strcpy(string, strtok(strstr(buffRead, "IP=")+3,":"));

		wiflyCMDModeOff();	//Surt del mode comandes
	}
	xSemaphoreGive(xMutexWifly);
}

/*****************************************************************************
 * Function:		wiflyGetChannel
 * Description:		Obté el canal en el que s'ha establert la connexió
 *
 * @return			Canal (int)
 *****************************************************************************/
uint32_t wiflyGetChannel(){

	uint32_t dada;

	xSemaphoreTake(xMutexWifly, portMAX_DELAY);
	{
		wiflyCMDModeOn();	//Entra en mode comandes

		UARTIOPrintTextCRLF(UART_WIFLY, "show net");
		UARTIOScanf(UART_WIFLY, buffRead, TIME_SHOW);
		dada = atoi(strtok(strstr(buffRead, "Chan=")+5,"\r\n"));

		wiflyCMDModeOff();	//Surt del mode comandes
	}
	xSemaphoreGive(xMutexWifly);

	return dada;
}

/*****************************************************************************
 * Function:		wiflyGetRSSI
 * Description:		Obté el RSSI del senyal (dBm)
 *
 * @return			RSSI (int)
 *****************************************************************************/
uint32_t wiflyGetRSSI(){

	uint32_t dada;

	xSemaphoreTake(xMutexWifly, portMAX_DELAY);
	{
		wiflyCMDModeOn();	//Entra en mode comandes

		UARTIOPrintTextCRLF(UART_WIFLY, "show rssi");
		UARTIOScanf(UART_WIFLY, buffRead, TIME_SHOW);
		dada = atoi(strtok(strstr(buffRead, "RSSI=(")+6,")"));

		wiflyCMDModeOff();	//Surt mode comandes
	}
	xSemaphoreGive(xMutexWifly);

	return dada;
}

/*****************************************************************************
 * Function:		wiflyGetUpTime
 * Description:		Obté el temps des de que el mòdul s'ha encès (segons)
 *
 * @return			UpTime (int)
 *****************************************************************************/
uint32_t wiflyGetUpTime(){

	uint32_t dada;

	xSemaphoreTake(xMutexWifly, portMAX_DELAY);
	{
		wiflyCMDModeOn();	//Entra en mode comandes

		UARTIOPrintTextCRLF(UART_WIFLY, "show time");
		UARTIOScanf(UART_WIFLY, buffRead, TIME_SHOW);
		dada = atoi(strtok(strstr(buffRead, "UpTime=")+7,"s"));

		wiflyCMDModeOff();	//Surt del mode comandes
	}
	xSemaphoreGive(xMutexWifly);

	return dada;
}

/*****************************************************************************
 * Function:		wiflyGetTxPower
 * Description:		Obté la potència de transmissió del mòdul (dBm)
 *
 * @return			TxPower (int)
 *****************************************************************************/
uint32_t wiflyGetTxPower(){

	uint32_t dada;

	xSemaphoreTake(xMutexWifly, portMAX_DELAY);
	{
		wiflyCMDModeOn();	//Entra en mode comandes

		UARTIOPrintTextCRLF(UART_WIFLY, "get wlan");
		UARTIOScanf(UART_WIFLY, buffRead, TIME_SHOW);
		dada = atoi(strtok(strstr(buffRead, "TxPower=")+8,"\r\n"));

		wiflyCMDModeOff();	//Surt del mode comandes
	}
	xSemaphoreGive(xMutexWifly);

	return dada;
}

/*****************************************************************************
 * Function:		wiflyGetTCPConfig
 * Description:		Obté les dades del servidor remot actualment configurat
 *
 * @param[out]:		IP		IP del host remot
 * @param[out]:		Host	Nom DNS del host remot
 * @param[out]:		Port	Port del host remot
 *
 * @return			None
 *****************************************************************************/
void wiflyGetTCPConfig(char *IP, char *Host, uint32_t *Port){

	xSemaphoreTake(xMutexWifly, portMAX_DELAY);
	{
		wiflyCMDModeOn();	//Entra en mode comandes

		UARTIOPrintTextCRLF(UART_WIFLY, "get ip");
		UARTIOScanf(UART_WIFLY, buffRead, TIME_GET);
		strcpy(IP, strtok(strstr(buffRead, "HOST=")+5,"\r\n"));

		*Port = atoi(strtok(strstr(IP, ":")+1,"\r\n"));
		strcpy(IP, strtok(IP,":"));

		UARTIOPrintTextCRLF(UART_WIFLY, "get dns");
		UARTIOScanf(UART_WIFLY, buffRead, TIME_GET);
		strcpy(Host, strtok(strstr(buffRead, "Name=")+5,"\n"));

		wiflyCMDModeOff();	//Surt del mode comandes
	}
	xSemaphoreGive(xMutexWifly);
}

/*****************************************************************************
 *                              SET FUNCTIONS
 *****************************************************************************/
/*****************************************************************************
 * Function:		wiflySetTxPower
 * Description:		Modifica la potència de transmissió del mòdul (dBm)
 *
 * @param[in]:		Power	Potència (dBm)
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
uint8_t wiflySetTxPower(uint32_t Power){

	uint8_t check = FALSE;

	xSemaphoreTake(xMutexWifly, portMAX_DELAY);
	{
		wiflyCMDModeOn();	//Entra en mode comandes

		sprintf(buffSend, "set wlan tx %i", Power);
		check = wiflySendCommandCRLF(buffSend, "AOK", TIME_SET);

		wiflyCMDModeOff();	//Surt del mode comandes
	}
	xSemaphoreGive(xMutexWifly);

	return check;
}

/*****************************************************************************
 * Function:		wiflySetTCPConfig
 * Description:		Configura el mòdul per a connexió TCP
 *
 * @param[in]:		HostType	Format en el que passem el host: Nom DNS(0) O IP(1)
 * @param[in]:		Host		IP o Nom DNS del host remot
 * @param[in]:		Port		Port del host remot
 *
 * @return			None
 *****************************************************************************/
void wiflySetTCPConfig(uint8_t HostType, char* Host, uint32_t Port){

	xSemaphoreTake(xMutexWifly, portMAX_DELAY);
	{
		wiflyCMDModeOn();	//Entra en mode comandes

		wiflySendCommandCRLF("set ip proto 2", "AOK", TIME_SET);	//TCP Server & Client (Default Mode)

		switch (HostType) {
		case 0:	//DNS
			sprintf(buffSend, "set dns name %s", Host);
			wiflySendCommandCRLF(buffSend, "AOK", TIME_SET);
			wiflySendCommandCRLF("set ip host 0", "AOK", TIME_SET);
			break;

		case 1:	//IP
			wiflySendCommandCRLF("set dns name 0", "AOK", TIME_SET);
			sprintf(buffSend, "set ip host %s", Host);
			wiflySendCommandCRLF(buffSend, "AOK", TIME_SET);
			break;
		}

		sprintf(buffSend, "set ip remote %i", Port);
		wiflySendCommandCRLF(buffSend, "AOK", TIME_SET);

		/* Sets the string that is sent to the local UART when the TCP port is opened */
		wiflySendCommandCRLF("set comm open *OPEN*", "AOK", TIME_SET);

		/* Sets the string that is sent to the remote TCP client when the TCP port is opened */
		wiflySendCommandCRLF("set comm remote 0", "AOK", TIME_SET);

		wiflyCMDModeOff();	//Surt del mode comandes
	}
	xSemaphoreGive(xMutexWifly);
}

/*****************************************************************************
 *                           TCP SOCKET FUNCTIONS
 *****************************************************************************/
/*****************************************************************************
 * Function:		wiflyTCPOpen
 * Description:		Obre una connexió TCP segons la configuració actual
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
uint8_t wiflyTCPOpen(){

	uint8_t check = FALSE;
	wiflyCMDModeOn();	//Entra en mode comandes
	check = wiflySendCommandCRLF("open", "*OPEN*", TIME_TCPOPEN);

	if (!check)
		wiflyCMDModeOff();	//Si no s'ha obert la connexió TCP, surt del mode comandes

	return check;
}

/*****************************************************************************
 * Function:		wiflyTCPSendDataSocket
 * Description:		Envia dades a través del socket
 *
 * @param[in]:		data		Dades a enviar pel socket
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
void wiflySendDataSocket(char* data){

	UARTIOPrintTextCRLF(UART_WIFLY, data);
}

/*****************************************************************************
 * Function:		wiflyTCPGetDataSocket
 * Description:		Rep dades a través del socket
 *
 * @param[out]:		data		Punter de la variable char on es guardaran
 * 								les dades rebudes
 * @param[out]:		timeout		Temps d'espera (ms) per rebre les dades
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
void wiflyGetDataSocket(char *data, uint32_t timeout){

	UARTIOScanf(UART_WIFLY, data, timeout);
}


/*****************************************************************************
 *                           PRIVATE FUNCTIONS
 *****************************************************************************/
/*****************************************************************************
 * Function:		wiflyCMDModeOn [PRIVATE FUNCTION]
 * Description:		Entra en mode comandes
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
uint8_t wiflyCMDModeOn(){

	uint8_t check = FALSE;
	check = wiflySendCommand("$$$", "CMD", TIME_SET);
	return check;
}

/*****************************************************************************
 * Function:		wiflyCMDModeOff [PRIVATE FUNCTION]
 * Description:		Surt del mode comandes
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
uint8_t wiflyCMDModeOff(){

	uint8_t check = FALSE;
	check = wiflySendCommandCRLF("exit", "EXIT", TIME_SET);
	return check;
}

/*****************************************************************************
 * Function:		wiflySendCommand	[PRIVATE FUNCTION]
 * Description:		Envia una comanda al Wifly i verifica la resposta
 *
 * @param[in]:		command		Comanda a enviar al Wifly
 * @param[in]:		success		Resposta esperada si l'execució ha estat safisfactòria
 * @param[in]:		timeout		Temps d'espera per llegir la resposta
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
uint8_t wiflySendCommand(char* command, char *success, uint32_t timeout){

	uint8_t check = FALSE;

	UARTIOPrintText(UART_WIFLY, command);
	UARTIOScanf(UART_WIFLY, buffRead, timeout);
	#ifdef DEBUG_WIFLY
		debug(buffRead);
	#endif

	if(strstr(buffRead, success))
		check = TRUE;

	return check;
}

/*****************************************************************************
 * Function:		wiflySendCommandCRLF	[PRIVATE FUNCTION]
 * Description:		Envia una comanda al Wifly, amb un salt de línia al final
 * 					i verifica la resposta
 *
 * @param[in]:		command		Comanda a enviar al Wifly
 * @param[in]:		success		Resposta esperada si l'execució ha estat safisfactòria
 * @param[in]:		timeout		Temps d'espera per llegir la resposta
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
uint8_t wiflySendCommandCRLF(char* command, char *success, uint32_t timeout){

	uint8_t check = FALSE;

	UARTIOPrintTextCRLF(UART_WIFLY, command);
	UARTIOScanf(UART_WIFLY, buffRead, timeout);
	#ifdef DEBUG_WIFLY
		debug(buffRead);
	#endif

	if(strstr(buffRead, success))
		check = TRUE;

	return check;
}
