/*****************************************************************************
 * File:			main.c
 * Description:		Fitxer amb el codi de l'aplicació principal
 * Project:			Sistema d'alarma i monitorització ambiental amb gestió remota
 * 					per a entorns crítics
 *
 * Version:			1.0		26/12/2013
 * Author:			Sergio Gómez
 *****************************************************************************/

/* FreeRTOS Includes */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "queue.h"

/* LPC Includes */
#include "LPC17xx.h"
#include "type.h"

/* Library Includes */
#include "config.h"
#include "debug.h"
#include "wifly.h"
#include "tmp102.h"
#include "commonUtils.h"
#include "comm.h"
#include "buzzer.h"

/* App Includes */
#include "appConfig.h"

#include <string.h>		//Required for: strlen
#include <stdio.h>		//Required for: sprintf
#include <stdlib.h>		//Required for: atoi, atof...

/* Task functions */
void vGetTempTask(void *pvParameters);
void vSendTempTask(void *pvParameters);
void vGetConfigTask(void *pvParameters);
void vAlarmTask(void *pvParameters);
void vBuzzerAlarmTask(void *pvParameters);

/* TaskHandle */
xTaskHandle xGetTempTaskHandle;
xTaskHandle xSendTempTaskHandle;
xTaskHandle xGetConfigTaskHandle;
xTaskHandle xAlarmTaskHandle;
xTaskHandle xBuzzerAlarmTaskHandle;

/* xQueueHandle */
xQueueHandle xQueueSensorToAlarm;
xQueueHandle xQueueAlarmToServer;

/* Private functions */
uint8_t appInit(void);
void sendMailAlarmOn(float Temp);
void sendMailAlarmOff(float Temp);

/* Variables */
Config ServerConfig;							//Variable que conté la configuració actualitzada

/* Dades configuració AP */
static char* WLAN_SSID = "WLAN_BINARIA_2";		//SSID de la xarxa
static char* WLAN_PWD = "OOBJJUKK";				//Contrasenya de la xarxa
static uint8_t WLAN_AUTH = WPA;					//Tipus d'autenticació de la xarxa: OPEN / WEP / WPA

/*Dades Servidor TCP*/
static char *ServerHost = "tfc.sergiogomez.es";	//Host Name or IP (ex: 'tfc.sergiogomez.es' or '192.168.1.18')
static int ServerPort = 6001;					//Port
static uint8_t HostType = HOSTDNS;				//HOSTDNS for Name or HOSTIP for IP


/*****************************************************************************
 * Function:		main
 * Description:		Inicialitza els perifèrics, crea les cues i les tasks, i
 * 					posa en marxa el planificador.
 *
 * @return			None
 *****************************************************************************/
int main( void )
{
	/* Inicialització dels perifèrics */
	while (!appInit());

	/* Creació de les cues */
	xQueueSensorToAlarm = xQueueCreate(QUEUE_SENSORTOALARM_SIZE, sizeof(float));
	xQueueAlarmToServer = xQueueCreate(QUEUE_ALARMTOSERVER_SIZE, sizeof(float));

	if ((xQueueSensorToAlarm != NULL) && (xQueueAlarmToServer != NULL)){

		/* Creació de les Taks */
		/* Task productor de temperatura */
		xTaskCreate(vGetTempTask, "Get Temp Task", GETTEMPTASK_STACK_SIZE, NULL, GETTEMPTASK_PRIORITY, &xGetTempTaskHandle);

		/* Task consumidor de temperatura */
		xTaskCreate(vSendTempTask, "Send Temp Task", SENDTEMPTASK_STACK_SIZE, NULL, SENDTEMPTASK_PRIORITY, &xSendTempTaskHandle);

		/* Task que obté la configuració del servidor */
		xTaskCreate(vGetConfigTask, "Get Config Task", GETCONFIGTASK_STACK_SIZE, NULL, GETCONFIGTASK_PRIORITY , &xGetConfigTaskHandle);

		/* Task que processa les dades de temperatura i gestiona l'estat d'alarma */
		xTaskCreate(vAlarmTask, "Alarm Task", ALARMTASK_STACK_SIZE, NULL, ALARMTASK_PRIORITY, &xAlarmTaskHandle);

		/* Task de l'alarma sonora */
		xTaskCreate(vBuzzerAlarmTask, "Alarm Task", BUZZERALARMTASK_STACK_SIZE, NULL, BUZZERALARMTASK_PRIORITY, &xBuzzerAlarmTaskHandle);
		vTaskSuspend(xBuzzerAlarmTaskHandle);	//Inicialment la task està apagada

		/* Inicia el planificador. Les taks comencen a executar-se */
		vTaskStartScheduler();
	}

	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
	return 0;
}

/*****************************************************************************
 *                                  TASKS
 *****************************************************************************/
/*****************************************************************************
 * Task Name:		vGetTempTask
 * Description:		Obté la temperatura del sensor i la posa a la cua xQueueSensorToAlarm.
 * 					La freqüència d'obtenció de les dades de temperatura ve donada
 *					per la configuració proporcionada pel servidor.
 *
 * @return			None
 *****************************************************************************/
void vGetTempTask( void *pvParameters )
{
	portBASE_TYPE xStatus;
	float Temp;

	#ifdef DEBUG_APP
		char buffDebug[BUFSENDMIN];
	#endif

	for( ;; )
	{
		/* Comprovem que la cua no estigui plena i que s'hagin obtingut les dades de configuració */
		if (((uxQueueMessagesWaiting(xQueueSensorToAlarm) <  QUEUE_SENSORTOALARM_SIZE))&&(ServerConfig.LastUpdate)){

			/* Obtenció de temperatura */
			Temp = tmp102GetTemp();

			#ifdef DEBUG_APP
				sprintf(buffDebug, "[vGetTempTask] Temperatura obtinguda: %f", Temp);
				debug(buffDebug);
			#endif

			/* Posem la temperatura obtinguda a la cua */
			xStatus = xQueueSendToBack(xQueueSensorToAlarm, &Temp, 0);

			if (xStatus != pdPASS){
				#ifdef DEBUG_APP
					debug("[vGetTempTask] Error enviant dada a la cua");
				#endif
			}

			/* Espera per obtenir la propera temperatura */
			vDelayMs(PASSIVE, ServerConfig.TempInterval*1000);	//Convert to ms

			#ifdef DEBUG_APP_STACK
				uint32_t stackHWM;
				stackHWM = uxTaskGetStackHighWaterMark(xGetTempTaskHandle);
				sprintf(buffDebug, "[vGetTempTask] Stack space available: %d", stackHWM);
				debug(buffDebug);
			#endif
		}
	}
}

/*****************************************************************************
 * Task Name:		vSendTempTask
 * Description:		Obté les dades de temperatura de la cua xQueueAlarmToServer i
 * 					les envia al servidor.
 *
 * @return			None
 *****************************************************************************/
void vSendTempTask( void *pvParameters )
{
	float Temp;
	portBASE_TYPE xStatus;
	portTickType xTicksToWait;

	#ifdef DEBUG_APP
		char buffDebug[BUFSENDMIN];
	#endif

	for ( ;; )
	{
		/* Comprovem que s'hagin obtingut les dades de configuració */
		if (ServerConfig.LastUpdate){

			/* Temps d'espera de la task en estat "blocked" a que hi hagin dades disponibles a la cua.
			 * El temps d'espera varia segons la freqüència d'obtenció de temperatures */
			xTicksToWait = ((ServerConfig.TempInterval * 1000)*SENDTEMPTASK_WAITTIME_MULT) / portTICK_RATE_MS;

			/* Recepció de dades de la cua */
			xStatus = xQueueReceive(xQueueAlarmToServer, &Temp, xTicksToWait);

			if (xStatus==pdPASS)
			{
				#ifdef DEBUG_APP
					sprintf(buffDebug, "[vSendTempTask] Dada rebuda de la cua: %f", Temp);
					debug(buffDebug);
				#endif

				/* Enviament de la temperatura al servidor */
				commSendTemp(Temp);

				#ifdef DEBUG_APP_STACK
					uint32_t stackHWM;
					stackHWM = uxTaskGetStackHighWaterMark(xSendTempTaskHandle);
					sprintf(buffDebug, "[vSendTempTask] Stack space available: %d", stackHWM);
					debug(buffDebug);
				#endif

			} else {
				#ifdef DEBUG_APP
					debug("[vSendTempTask] ERROR obtenint dada de la cua");
				#endif
			}
		}
	}
}

/*****************************************************************************
 * Task Name:		vGetConfigTask
 * Description:		Comprova si el servidor disposa de configuració més recent
 * 					i, en cas afirmatiu, actualitza les dades de configuració
 * 					de l'aplicació.
 *
 * @return			None
 *****************************************************************************/
void vGetConfigTask( void *pvParameters )
{
	int lastUpdateServer;
	int updateInterval = UPDATE_INTERVAL;

	#ifdef DEBUG_APP
		char buffDebug[BUFSENDMIN];
	#endif

	for ( ;; )
	{
		/* Obtenció de la data de la darrera actualització de la configuració de que disposa el servidor */
		lastUpdateServer = commGetLastUpdate();

		if (lastUpdateServer > ServerConfig.LastUpdate ){
			/* El servidor disposa de configuració més recent */
			#ifdef DEBUG_APP
				debug("[vGetConfigTask] El servidor disposa de configuracio mes recent");
			#endif

			/* Actualitzem amb la nova configuració */
			commUpdateConfig(&ServerConfig);

			/* Establim quan serà la propera actualització */
			/* L'interval serà el mateix que el definit per l'usuari per l'obtenció de temperatures, sense sobrepassar el definit per UPDATE_INTERVAL */
			if (ServerConfig.TempInterval < UPDATE_INTERVAL){
				updateInterval = ServerConfig.TempInterval;
			} else {
				updateInterval = UPDATE_INTERVAL;
			}

		} else {
			#ifdef DEBUG_APP
				debug("[vGetConfigTask] No cal actualitzar");
			#endif
		}

		#ifdef DEBUG_APP
			debugNum("[vGetConfigTask] La propera actualitzacio sera en (s):",updateInterval);
		#endif

		#ifdef DEBUG_APP_STACK
			uint32_t stackHWM;
			stackHWM = uxTaskGetStackHighWaterMark(xGetConfigTaskHandle);
			sprintf(buffDebug, "[vGetConfigTask] Stack space available: %d", stackHWM);
			debug(buffDebug);
		#endif

		/* Espera per a la propera actualització */
		vDelayMs(PASSIVE, updateInterval*1000); //Convert to ms
	}
}

/*****************************************************************************
 * Task Name:		vAlarmTask
 * Description:		Obté les dades de temperatura de la cua xQueueSensorToAlarm,
 * 					activa o desactiva l'alarma, i posa les dades a la cua
 * 					xQueueAlarmToServer.
 *
 * @return			None
 *****************************************************************************/
void vAlarmTask( void *pvParameters )
{
	float Temp;
	portBASE_TYPE xStatus;
	portTickType xTicksToWait;
	uint8_t alarmStatus = FALSE;		//Variable per controlar l'estat d'alarma

	#ifdef DEBUG_APP
		char buffDebug[BUFSENDMIN];
	#endif

	for ( ;; )
	{
		/* Comprovem que s'hagin obtingut les dades de configuració */
		if (ServerConfig.LastUpdate){

			/* Temps d'espera de la task en estat "blocked" a que hi hagin dades disponibles a la cua.
			 * El temps d'espera varia segons la freqüència d'obtenció de temperatures */
			xTicksToWait = ((ServerConfig.TempInterval * 1000)*ALARMTASK_WAITTIME_MULT) / portTICK_RATE_MS;

			/* Obtenció de temperatura de la cua */
			xStatus = xQueueReceive(xQueueSensorToAlarm, &Temp, xTicksToWait);

			if (xStatus==pdPASS)
			{
				#ifdef DEBUG_APP
					sprintf(buffDebug, "[vAlarmTask] Dada rebuda de la cua: %f", Temp);
					debug(buffDebug);
				#endif

				/** INICI PROCESSAT DE LA DADA DE TEMPERATURA **/
				/***********************************************/
				if (alarmStatus == FALSE){
					/* ALARMA ACTUALMENT DESACTIVADA */

					if ((Temp < ServerConfig.TempMin) || (Temp > ServerConfig.TempMax)){
						/* TEMPERATURA FORA DELS LÍMITS DE SEGURETAT */
						/* 1. Activem estat d'alarma */
						alarmStatus = TRUE;
						#ifdef DEBUG_APP
							debug("[vAlarmTask] ALARMA ACTIVADA");
						#endif

						/* 2. Activem buzzer si està activat per configuració */
						if (ServerConfig.SoundAlarm==TRUE)
							vTaskResume(xBuzzerAlarmTaskHandle);

						/* 3. Enviem mail d'alerta */
						sendMailAlarmOn(Temp);
					}
				} else {
					/* ALARMA ACTUALMENT ACTIVADA */

					if ((Temp > ServerConfig.TempMin + ALARMOFF_THRESHOLD) && (Temp < ServerConfig.TempMax - ALARMOFF_THRESHOLD)){
						/* TEMPERATURA DINS DELS LIMITS DE SEGURETAT */
						/* 1. Desactivem estat d'alarma */
						alarmStatus = FALSE;
						#ifdef DEBUG_APP
							debug("[vAlarmTask] ALARMA DESACTIVADA");
						#endif

						/* 2. Aturem Buzzer si estava sonant */
						if(!xTaskIsTaskSuspended(xBuzzerAlarmTaskHandle)) {
							vTaskSuspend(xBuzzerAlarmTaskHandle);
							buzzerOff();
						}
						/* 3. Enviem notificació per mail */
						sendMailAlarmOff(Temp);

					} else {
						/* LA TEMPERATURA CONTINUA FORA DELS MARGES DE SEGURETAT */
						/* Revisem si la configuració del buzzer ha canviat */
						if  ((ServerConfig.SoundAlarm==FALSE) && (!xTaskIsTaskSuspended(xBuzzerAlarmTaskHandle))){
							vTaskSuspend(xBuzzerAlarmTaskHandle);
							buzzerOff();
						} else if ((ServerConfig.SoundAlarm==TRUE) && (xTaskIsTaskSuspended(xBuzzerAlarmTaskHandle))){
							vTaskResume(xBuzzerAlarmTaskHandle);
						}
					}
				}
				/**  FI PROCESSAT DE LA DADA DE TEMPERATURA   **/
				/***********************************************/

				/* Comprovem que la cua no estigui plena */
				if ((uxQueueMessagesWaiting(xQueueAlarmToServer) < QUEUE_ALARMTOSERVER_SIZE) ){

					/* Posem la dada obtinguda a la cua */
					xStatus = xQueueSendToBack(xQueueAlarmToServer, &Temp, 0);

					if (xStatus != pdPASS ){
						#ifdef DEBUG_APP
							debug("[vAlarmTask] Error enviant dada a la cua");
						#endif
					}
				}

				#ifdef DEBUG_APP_STACK
					uint32_t stackHWM;
					stackHWM = uxTaskGetStackHighWaterMark(xAlarmTaskHandle);
					sprintf(buffDebug, "[vAlarmTask] Stack space available: %d", stackHWM);
					debug(buffDebug);
				#endif

			} else {
				#ifdef DEBUG_APP
					debug("[vAlarmTask] ERROR obtenint dada de la cua");
				#endif
			}
		}
	}
}

/*****************************************************************************
 * Task Name:		vBuzzerAlarmTask
 * Description:		Activa l'alarma sonora (Buzzer)
 *
 * @return			None
 *****************************************************************************/
void vBuzzerAlarmTask( void *pvParameters )
{
	for ( ;; )
	{
		buzzerToggle();
		vDelayMs(PASSIVE, 1000);
	}
}


/*****************************************************************************
 *                           PRIVATE FUNCTIONS
 *****************************************************************************/
/*****************************************************************************
 * Function:		sendMailAlarmOn
 * Description:		Envia un mail alertant de que la temperatura està fora dels
 * 					marges de seguretat establerts.
 *
 * @param[in]		Temp	Temperatura
 *
 * @return			None
 *****************************************************************************/
void sendMailAlarmOn(float Temp)
{

	char *MailSubject = "Alarma de temperatura";	//Assumpte del mail
	char TempFormatted[BUFSENDMIN];
	char MailBody[BUFSENDMIN];

	sprintf(TempFormatted, FORMAT_SEND_TEMP, Temp);
	sprintf(MailBody, "La temperatura esta fora dels marges de seguretat. Temperatura obtinguda: %s graus", TempFormatted);

	#ifdef DEBUG_APP
		debug("[sendMailAlarmOn] Enviant mail...");
	#endif

	if (commSendMail(ServerConfig.MailServer, ServerConfig.MailPort, ServerConfig.MailUser, ServerConfig.MailPass, ServerConfig.MailTo, MailSubject, MailBody)){
		#ifdef DEBUG_APP
			debug("[sendMailAlarmOn] Mail enviat");
		#endif
	} else {
		#ifdef DEBUG_APP
			debug("[sendMailAlarmOn] Error enviant mail");
		#endif
	}
}

/*****************************************************************************
 * Function:		sendMailAlarmOff
 * Description:		Envia un mail informant de que la temperatura torna a estar
 * 					dins dels marges de seguretat establerts.
 *
 * @param[in]		Temp	Temperatura
 *
 * @return			None
 *****************************************************************************/
void sendMailAlarmOff(float Temp)
{

	char *MailSubject = "Temperatura reestablerta";		//Assumpte del mail
	char TempFormatted[BUFSENDMIN];
	char MailBody[BUFSENDMIN];

	sprintf(TempFormatted, FORMAT_SEND_TEMP, Temp);
	sprintf(MailBody, "La temperatura esta dins dels marges de seguretat. Temperatura obtinguda: %s graus.", TempFormatted);

	#ifdef DEBUG_APP
		debug("[sendMailAlarmOff] Enviant mail...");
	#endif

	if (commSendMail(ServerConfig.MailServer, ServerConfig.MailPort, ServerConfig.MailUser, ServerConfig.MailPass, ServerConfig.MailTo, MailSubject, MailBody)){
		#ifdef DEBUG_APP
			debug("[sendMailAlarmOff] Mail enviat");
		#endif
	} else {
		#ifdef DEBUG_APP
			debug("[sendMailAlarmOff] Error enviant mail");
		#endif
	}
}

/*****************************************************************************
 * Function:		appInit
 * Description:		Inicialitza els perifèrics (Buzzer, CP2102, TMP102 i Wifly),
 * 					estableix la configuració de zero del WiFly, es connecta a
 * 					la xarxa i configura les dades del servidor.
 *
 * @return			TRUE en cas d'èxit o FALSE en cas d'error
 *****************************************************************************/
uint8_t appInit(void)
{
	uint8_t check = FALSE;

	buzzerInit();		//Inicialitza Buzzer
	debugInit();		//Inicialitza mòdul de Debug (CP2102)
	tmp102Init();		//Inicialitza sensor TMP102

	#ifdef DEBUG_APP
		debug("[appInit] Inicialitzant WiFly...");
	#endif

	wiflyInit(WIFLY_BAUDRATE);	//Inicialitza el WiFly

	#ifdef DEBUG_APP
		debug("[appInit] Connectant a la SSID...");
	#endif

	if (wiflyConnectSSID(WLAN_SSID, WLAN_AUTH, WLAN_PWD)){			//Connexió a la xarxa
		/* La connexió a la SSID s'ha realitzat correctament */
		#ifdef DEBUG_APP
			debug("[appInit] Connexio a la SSID OK");
			debug("[appInit] Configurant Servidor TCP per primer cop...");
		#endif
		commSrvInit(HostType, ServerHost, ServerPort);
		check = TRUE;

	} else {
		#ifdef DEBUG_APP
			debug("[appInit] ERROR connectant a la SSID");
		#endif
	}

	vDelayMs(ACTIVE, 1000);		//Petita espera activa després de totes les inicialitzacions, per evitar errors

	return check;
}


/*****************************************************************************
 *                             HOOK FUNCTIONS
 *****************************************************************************/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	for( ;; );
}

/*---------------------------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	debug("[ERROR] Stack Overflow");
	debug(pcTaskName);
	for( ;; );
}

/*---------------------------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This project does not use the idle hook to perform any processing. */
}

/*---------------------------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This projects does not use the tick hook to perform any processing. */
}
